@@ -6,6 +6,7 @@
 #include <libart_lgpl/art_affine.h>
 
 #include <string.h>
+#include <libgnomeprint/gp-unicode.h>
 #include <libgnomeprint/gnome-font.h>
 #include <libgnomeprint/gnome-font-private.h>
 
@@ -36,26 +37,19 @@ enum {
 GtkType
 gnome_font_get_type (void)
 {
-  static GtkType font_type = 0;
-
-  if (!font_type)
-    {
-      GtkTypeInfo font_info =
-      {
-	"GnomeFont",
-	sizeof (GnomeFont),
-	sizeof (GnomeFontClass),
-	(GtkClassInitFunc) gnome_font_class_init,
-	(GtkObjectInitFunc) gnome_font_init,
-	/* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
-      };
-
-      font_type = gtk_type_unique (gtk_object_get_type (), &font_info);
-    }
-
-  return font_type;
+	static GtkType font_type = 0;
+	if (!font_type) {
+		GtkTypeInfo font_info = {
+			"GnomeFont",
+			sizeof (GnomeFont),
+			sizeof (GnomeFontClass),
+			(GtkClassInitFunc) gnome_font_class_init,
+			(GtkObjectInitFunc) gnome_font_init,
+			NULL, NULL, NULL
+		};
+		font_type = gtk_type_unique (gtk_object_get_type (), &font_info);
+	}
+	return font_type;
 }
 
 #define KERN_PAIR_HASH(g1, g2) ((g1) * 367 + (g2) * 31)
@@ -528,12 +522,44 @@ gnome_font_get_face (const GnomeFont * font)
 gdouble
 gnome_font_get_glyph_width (const GnomeFont *font, gint ch)
 {
-  g_return_val_if_fail (font != NULL, 0.0);
-  g_return_val_if_fail (GNOME_IS_FONT (font), 0.0);
+	g_return_val_if_fail (font != NULL, 0.0);
+	g_return_val_if_fail (GNOME_IS_FONT (font), 0.0);
 
-  if (ch < 0 || ch >= 256) return 0.0;
+	if (ch < 0 || ch >= 256) return 0.0;
 
-  return gnome_font_face_get_glyph_width (font->private->fontmap_entry, ch) * font->private->scale;
+	return gnome_font_face_get_glyph_width (font->private->fontmap_entry, ch) * font->private->scale;
+}
+
+double
+gnome_font_get_width_utf8 (const GnomeFont *font, const char *s)
+{
+	g_return_val_if_fail (font != NULL, 0.0);
+	g_return_val_if_fail (GNOME_IS_FONT (font), 0.0);
+	g_return_val_if_fail (s != NULL, 0.0);
+
+	return gnome_font_get_width_utf8_sized (font, s, strlen (s));
+}
+
+double
+gnome_font_get_width_utf8_sized (const GnomeFont *font, const char *text, int n)
+{
+	double width;
+	const gchar *p;
+
+	g_return_val_if_fail (font != NULL, 0.0);
+	g_return_val_if_fail (GNOME_IS_FONT (font), 0.0);
+	g_return_val_if_fail (text != NULL, 0.0);
+
+	width = 0.0;
+
+	for (p = text; p && p < (text + n); p = g_utf8_next_char (p)) {
+		gint unival, glyph;
+		unival = g_utf8_get_char (p);
+		glyph = gnome_font_lookup_default (font, unival);
+		width += gnome_font_face_get_glyph_width (font->private->fontmap_entry, glyph);
+	}
+
+	return width * font->private->scale;
 }
 
 /**
@@ -550,11 +576,11 @@ gnome_font_get_glyph_width (const GnomeFont *font, gint ch)
 double
 gnome_font_get_width_string (const GnomeFont *font, const char *s)
 {
-  g_return_val_if_fail (font != NULL, 0.0);
-  g_return_val_if_fail (GNOME_IS_FONT (font), 0.0);
-  g_return_val_if_fail (s != NULL, 0.0);
+	g_return_val_if_fail (font != NULL, 0.0);
+	g_return_val_if_fail (GNOME_IS_FONT (font), 0.0);
+	g_return_val_if_fail (s != NULL, 0.0);
 
-  return gnome_font_get_width_string_n (font, s, strlen (s));
+	return gnome_font_get_width_string_n (font, s, strlen (s));
 }
 
 /**
@@ -573,22 +599,22 @@ gnome_font_get_width_string (const GnomeFont *font, const char *s)
 double
 gnome_font_get_width_string_n (const GnomeFont *font, const char *s, int n)
 {
-  double width;
-  gint i;
+	double width;
+	gint i;
 
-  g_return_val_if_fail (font != NULL, 0.0);
-  g_return_val_if_fail (GNOME_IS_FONT (font), 0.0);
-  g_return_val_if_fail (s != NULL, 0.0);
+	g_return_val_if_fail (font != NULL, 0.0);
+	g_return_val_if_fail (GNOME_IS_FONT (font), 0.0);
+	g_return_val_if_fail (s != NULL, 0.0);
 
-  width = 0.0;
+	width = 0.0;
 
-  for (i = 0; i < n; i++) {
-	  gint glyph;
-	  glyph = gnome_font_face_lookup_default (font->private->fontmap_entry, s[i]);
-	  width += gnome_font_face_get_glyph_width (font->private->fontmap_entry, glyph);
-  }
+	for (i = 0; i < n; i++) {
+		gint glyph;
+		glyph = gnome_font_face_lookup_default (font->private->fontmap_entry, s[i]);
+		width += gnome_font_face_get_glyph_width (font->private->fontmap_entry, glyph);
+	}
 
-  return width * font->private->scale;
+	return width * font->private->scale;
 }
 
 /* Get the glyph number corresponding to a given unicode, or -1 if it
@@ -596,10 +622,10 @@ gnome_font_get_width_string_n (const GnomeFont *font, const char *s, int n)
 gint
 gnome_font_lookup_default (const GnomeFont *font, gint unicode)
 {
-  g_return_val_if_fail (font != NULL, -1);
-  g_return_val_if_fail (GNOME_IS_FONT (font), -1);
+	g_return_val_if_fail (font != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_FONT (font), -1);
 
-  return gnome_font_face_lookup_default (font->private->fontmap_entry, unicode);
+	return gnome_font_face_lookup_default (font->private->fontmap_entry, unicode);
 }
 
 static gboolean
