@@ -47,10 +47,11 @@
 #include <config.h>
 #include <stdio.h>
 #include <math.h>
+#include <libgnomeprint/gnome-printer.h>
 #include <libgnomeprint/gnome-printer-private.h>
 #include <libgnomeprint/gnome-print-pclv.h>
 #include <libgnomeprint/gnome-print-pclc.h>
-#include <libgnomeprint/gnome-print-encode.h>
+#include <libgnomeprint/gnome-print-encode-private.h>
 
 PCLJobData * jobdata;
 static GnomePrintRGBPClass *pclv_parent_class;
