@@ -188,6 +188,11 @@ static int gnome_print_ps2_image (GnomePrintContext *pc, const char *data, int w
 
 static gchar* gnome_print_ps2_get_date (void);
 
+
+#ifdef ENABLE_LIBGPA
+static gint gnome_print_ps2_get_level (GnomePrintPs2 *ps2);
+#endif
+
 static GnomePrintContextClass *parent_class;
 
 GtkType
@@ -584,7 +589,7 @@ gp_ps2_set_color_private (GnomePrintContext *pc, gdouble r, gdouble g, gdouble b
 
 	ps2 = (GnomePrintPs2 *) pc;
 
-	if ((ps2->private_font_flag == GP_GC_FLAG_CLEAR) && (r == ps2->r) && (g == ps2->g) && (b == ps2->b)) return 0;
+	if ((ps2->private_color_flag == GP_GC_FLAG_CLEAR) && (r == ps2->r) && (g == ps2->g) && (b == ps2->b)) return 0;
 
 	ret = gnome_print_context_fprintf (pc, "%.3g %.3g %.3g rg" EOL, r, g, b);
 
@@ -1199,11 +1204,10 @@ gnome_print_ps2_setfont (GnomePrintContext *pc, GnomeFont *font)
 #endif
 
 
-#if 0
+#ifdef ENABLE_LIBGPA
 static gint
 gnome_print_ps2_get_level (GnomePrintPs2 *ps2)
 {
-#ifdef ENABLE_LIBGPA
 	const gchar *temp;
 
 	g_return_val_if_fail (GNOME_IS_PRINT_PS2 (ps2), 2);
@@ -1225,8 +1229,7 @@ gnome_print_ps2_get_level (GnomePrintPs2 *ps2)
 		return 3;
 	else
 		g_warning ("Invalid PS Level %s", temp);
-#endif
-
+	
 	return 2;
 }
 #endif
