@@ -579,13 +579,13 @@ gnome_print_show_sized (GnomePrintContext *pc, char const *text, int bytes)
 	font = gp_gc_get_font (pc->gc);
 	if (!font) return -1;
 
-	if (PRINT_CLASS (pc)->glyphlist) {
+	if (PRINT_CLASS (pc)->show_sized) {
+		return (* PRINT_CLASS (pc)->show_sized) (pc, text, bytes);
+	} else if (PRINT_CLASS (pc)->glyphlist) {
 		GnomeGlyphList *gl;
 		gl = gnome_glyphlist_from_text_sized_dumb ((GnomeFont *) font, gp_gc_get_rgba (pc->gc), 0.0, 0.0, text, bytes);
 		gnome_print_glyphlist (pc, gl);
 		gnome_glyphlist_unref (gl);
-	} else if (PRINT_CLASS (pc)->show_sized) {
-		return (* PRINT_CLASS (pc)->show_sized) (pc, text, bytes);
 	}
 
 	return GNOME_PRINT_OK;
