@@ -882,7 +882,7 @@ gpa_options_dup_path (const GpaOptions *options)
 		if (options->parent->parent->parent != NULL) {
 			gpa_error ("We only support 1st & 2nd level option");
 		}
-		path  = g_strdup_printf ("%s-%s-%s",
+		path  = g_strdup_printf ("%s" GPA_PATH_DELIMITER "%s" GPA_PATH_DELIMITER "%s",
 					 options->parent->parent->id,
 					 options->parent->id,
 					 options->id);
