@@ -55,7 +55,9 @@ static struct poptOption options [] = {
 	{NULL}
 };
 
+#if 0
 static gint latin_to_utf8 (guchar * text, guchar * utext, gint ulength);
+#endif
 
 static gint print_paragraph (GnomePrintContext *pc, const guchar *b, const guchar *e, gdouble x0, gdouble *y0, gdouble x1, gdouble y1)
 {
@@ -364,7 +366,10 @@ do_rbuf (gboolean alpha, gboolean frgba)
 	p2b[5] = PMH;
 
 	buf = g_new (guchar, PMW * PMH * bpp);
+#warning I disabled this code casue it was throwing an error at me
+#if 0	
 	memset (buf, 0xff, PMW * PMH * bpp);
+#endif
 
 	pc = gnome_print_rbuf_new (buf, PMW, PMH, bpp * PMW, p2b, alpha);
 
@@ -433,6 +438,7 @@ main (int argc, char ** argv)
 	return 0;
 }
 
+#if 0
 static gint
 latin_to_utf8 (guchar * text, guchar * utext, gint ulength)
 {
@@ -446,4 +452,5 @@ latin_to_utf8 (guchar * text, guchar * utext, gint ulength)
 
 	return o - utext;
 }
+#endif
 
